/** 
 ****************************************************************************
 *
 *
 * @file    ComponentA.cpp
 *
 * @author  Charles Halliday
 *
 * @date    31 Mar 2008
 *
 ****************************************************************************
 */

#include "ComponentA.hpp"
#include "rtl/ustring.hxx"

#include <unistd.h>

#include <iostream>
#include <string>

using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
using namespace ::rtl;
using namespace ::com::myco;
using namespace ::std;


// Use the following editable section for
// shared variables in anonymous namespace,
// additional using namespace statements

namespace {
  using namespace ::com_myco_ComponentA;

  string toStr(OUString ouString)
  {
    string str(OUStringToOString(ouString,
                                 RTL_TEXTENCODING_ASCII_US).getStr());
    return str;
  }  // ! toStr

}

namespace com_myco_ComponentA {

  ComponentA::ComponentA(
    Reference< XComponentContext > const & xComponentContext)
    : ComponentABase::ComponentABase(xComponentContext)
    , mInitialized(sal_False)
  {
  }

  ComponentA::~ComponentA()
  {
  }

  // Interface com.sun.star.lang.XInitialization

  // Method of com.sun.star.lang.XInitialization
  void SAL_CALL
  ComponentA::initialize(
    const Sequence< Any >& aArguments )
    throw (
      Exception,
      RuntimeException )
  {
    ::osl::MutexGuard guard(mInitializedMutex);
    if ( mInitialized == sal_False ) {
      mInitialized = sal_True;
      DEBUG_LOG("called");
    }
  }

  // XMain
  sal_Int32 ComponentA::run(
    const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aArguments )
    throw (::com::sun::star::uno::RuntimeException)
  {
    DEBUG_LOG("Called");
    return 0;
  }

  // ZInterfaceA
  void  ComponentA::setObject( const ::rtl::OUString& objectName )
    throw (::com::sun::star::uno::RuntimeException)
  {
    INFO_LOG("Called");
    if (mObjects.find(objectName) != mObjects.end() ) {
      mObjects[objectName].isSet = sal_True;
      if ( mObjects[objectName].listener.is() ) {
        DEBUG_LOG("Calling newObjectAvailable(\""
                  << toStr(objectName) << "\")"
                  << ", number=" << mObjects.size());
        mObjects[objectName].listener->newObjectAvailable(objectName);
        mObjects[objectName].listener.clear();
      }
    }
    DEBUG_LOG("m_RefCount=" << m_refCount);
  }


  void  ComponentA::prepareObject(
    const ::rtl::OUString& objectName ,
    const ::com::sun::star::uno::Reference< ::com::myco::ZInterfaceAListener >& aListener )
    throw (::com::sun::star::uno::RuntimeException)
  {
    INFO_LOG("Called");
    struct ObjectInfo objectInfo;
    objectInfo.listener  = aListener;
    objectInfo.isSet     = sal_False;
    mObjects[objectName] = objectInfo;
  }


} // namespace close

