/** 
 ****************************************************************************
 *
 * Service notifying updates
 *
 * @file    TestComponentABase.cpp
 *
 * @author  Charles Halliday
 *
 * @date    31 Mar 2008
 *
 ****************************************************************************
 */

// Include own header
#include "TestComponentABase.hpp"

#include <iostream>

#include <cppuhelper/factory.hxx>
#include <cppuhelper/implementationentry.hxx>

#include <rtl/ustring.hxx>

// Service implementation headers (required for factory methods)
#include "TestComponentA.hpp"

using namespace ::std;
using namespace ::rtl;
using namespace ::cppu;
using namespace ::com::sun::star;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;

namespace {

  // Singleton reference, cleared in dispose
  Reference < XInterface > sSelf;
  ::osl::Mutex sDisposeMutex;

}

namespace com_myco_TestComponentA {

  // Implementation of service: com.myco.TestComponentA

  static const OUString __SERVICE_NAME_TestComponentA =
  OUString::createFromAscii("com.myco.TestComponentA");
  static const OUString __IMPLEMENTATION_NAME_TestComponentA =
  OUString::createFromAscii("com_myco_TestComponentA.TestComponentA");

  /** Factory
   * @param xComponentContext component context from client
   */
  Reference< XInterface > SAL_CALL create_TestComponentA(
    Reference< XComponentContext > const & xComponentContext )
    SAL_THROW( () )
  {
    ::osl::MutexGuard guard(sDisposeMutex);
    if ( ! sSelf.is() ) {
      TestComponentA * self = new TestComponentA(xComponentContext);
      sSelf =  Reference < XInterface >(
        static_cast< ::cppu::OWeakObject * > (self)) ;
      DEBUG_LOG("Returning new instance");
    }
    return sSelf;
  }

  /** 
   * @return all service names that may be used to access this service
   */
  static Sequence< OUString > getSupportedServiceNames_TestComponentA()
  {
    static Sequence < OUString > *pNames = 0;
    if( ! pNames )
    {
      static Sequence< OUString > seqNames(1);
      seqNames.getArray()[0] = OUString(__SERVICE_NAME_TestComponentA);
      pNames = &seqNames;
    }
    return *pNames;
  }

  /** 
   * @return implementation name
   */
  static OUString getImplementationName_TestComponentA()
  {
    return OUString(__IMPLEMENTATION_NAME_TestComponentA);
  }

  // Base Class Constructor
  TestComponentABase::TestComponentABase(
    Reference< XComponentContext > const & xComponentContext)
    : TestComponentAHelper(mBaseMutex)
  {
  }

  // Base Class Destructor
  TestComponentABase::~TestComponentABase()
  {
  }

  // XComponent
  void SAL_CALL TestComponentABase::dispose(  )
    throw (::com::sun::star::uno::RuntimeException)
  {
    ::osl::MutexGuard guard(sDisposeMutex);
    TestComponentAHelper::dispose();
    DEBUG_LOG("m_refCount=" << m_refCount);
    if ( sSelf.is() ) {
      if ( m_refCount > 1 ) {
        sSelf.clear();
      }
    }
  }
  // XMain base implementation
  /**
   * @param aArguments arguments passed to the component,
   *        e.g. the command line arguments
   * @return status: 0 for success, otherwise component dependent error code
   */
  sal_Int32 TestComponentABase::run(
    const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aArguments )
    throw (::com::sun::star::uno::RuntimeException)
  {
    // Override
    return 0;
  }

  // XServiceInfo base implementation

  /**
    * @return unique name of the implementation
    */
  ::rtl::OUString TestComponentABase::getImplementationName()
    throw (RuntimeException)
  {
    return getImplementationName_TestComponentA();
  }  // ! getImplementationName

  /**
    * @param full name of service to be tested
    *
    * @return true, if service is supported, false otherwise
    */
  sal_Bool TestComponentABase::supportsService( OUString const & serviceName )
    throw (RuntimeException)
  {
    sal_Bool bSupportService = sal_False;
    Sequence< OUString > supportedServices = getSupportedServiceNames();
    for ( sal_Int32 i = 0; i < supportedServices.getLength(); i++) {
      if (serviceName.equals(supportedServices[i])) {
        bSupportService = sal_True;
        break;
      }
    }
    return bSupportService;
  }  // ! supportsService

  /**
    *  Provides the supported service names of the implementation
    *
    * @return service names that are supported
    */
  Sequence< OUString > TestComponentABase::getSupportedServiceNames()
    throw (RuntimeException)
  {
    return getSupportedServiceNames_TestComponentA();
  }

  // end service com.myco.TestComponentA

  // Table for UNO component implementation helper functions
  static struct ImplementationEntry s_component_entries [] =
    {
      // Table entries for helper classes.
      // One entry per service implementation
      {
        create_TestComponentA,
        getImplementationName_TestComponentA,
        getSupportedServiceNames_TestComponentA,
        createSingleComponentFactory,
        0,
        0,
      },
      { 0, 0, 0, 0, 0, 0 }
    };

} // namespace close



// Standard C functions required by UNO for component
extern "C"
{

  void SAL_CALL component_getImplementationEnvironment(
    sal_Char const ** ppEnvTypeName, uno_Environment ** ppEnv )
  {
    *ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
  }

  sal_Bool SAL_CALL component_writeInfo(lang::XMultiServiceFactory * xMgr,
                                        registry::XRegistryKey * xRegistry )
  {
    // Table driven helper in cppuhelper
    return component_writeInfoHelper(xMgr,
                                     xRegistry,
                                     ::com_myco_TestComponentA::s_component_entries );
  }

  void * SAL_CALL component_getFactory(
    sal_Char const * implName, lang::XMultiServiceFactory * xMgr,
    registry::XRegistryKey * xRegistry )
  {
    // Table driven helper in cppuhelper
    DEBUG_LOG("");
    return component_getFactoryHelper(implName,
                                      xMgr,
                                      xRegistry,
                                      ::com_myco_TestComponentA::s_component_entries );
  }

}
