#!/bin/ksh

# Create SystemLooger configurtion file
# $1 Target file path
function createSystemLoggerConf {

cat <<EOF > $1
log4j.threshold=debug
log4j.rootLogger=debug, R, System
log4j.appender.R=org.apache.log4j.FileAppender
log4j.appender.R.Append=true
log4j.appender.R.File=\${XOC_SYSTEM}/log/SessionManagement.slg
log4j.appender.R.layout=org.apache.log4j.PatternLayout
log4j.appender.R.layout.ConversionPattern=%d{ABSOLUTE} %p %c %l %t %m%n

log4j.appender.System=org.apache.log4j.FileAppender
log4j.appender.System.Append=true
log4j.appender.System.File=\${XOC_SYSTEM}/log/CompleteSystem.slg
log4j.appender.System.layout=org.apache.log4j.PatternLayout
log4j.appender.System.layout.ConversionPattern=%d{ABSOLUTE} %p %c %l %t %m%n

EOF

}

# Create unorc file for TestComponentA
# $1 Target file path
function createUnorc {

cat <<EOF > $1
[Bootstrap]
UNO_TYPES=file://$OO_SDK_URE_HOME/share/misc/types.rdb file://$XOC_SYSTEM/cache/testTypes.rdb
UNO_SERVICES=file://$OO_SDK_URE_HOME/share/misc/services.rdb file://$XOC_SYSTEM/cache/testServices.rdb
EOF
}

# Create rdb containing all the services required for test
# $1 rdb directory path
function createTestRdbs {
  typeset -r rbdDir=$1
  typeset -r servicesRdb=$rbdDir/testServices.rdb
  typeset -r typesRdb=$rbdDir/testTypes.rdb

  typeset ll
  for ll in $WORKSPACE/services/session/SocketAcceptor/libSocketAcceptor.so $BASE/lib/*.uno.so ; do
    $OO_SDK_URE_BIN_DIR/regcomp \
      -register \
      -br file://$OO_SDK_URE_HOME/share/misc/services.rdb \
      -br file://$OO_SDK_URE_HOME/share/misc/types.rdb \
      -r  file://$rbdDir/testServices.rdb \
      -c  file://$ll \
      > /dev/null 2>&1
    exitCode=$?
    if [ $exitCode -ne 0 ] ; then
      echo "regcomp could not register $ll in $servicesRdb" 1>&2
    fi
  done

  $OO_SDK_URE_BIN_DIR/regmerge \
    $rbdDir/testTypes.rdb \
    /UCR $BASE/misc/ComponentA/com/myco/*.urd \
    > /dev/null 2>&1
    exitCode=$?
    if [ $exitCode -ne 0 ] ; then
      echo "regmerge could not register urds in $typesRdb" 1>&2
    fi

}

# Creates TestBed under $XOC_SYSTEM
function createTestBed {
  if [ -d "$XOC_SYSTEM" ] ; then
    rm -fr $XOC_SYSTEM
  fi

  mkdir $XOC_SYSTEM
  mkdir $XOC_SYSTEM/bin
  mkdir $XOC_SYSTEM/etc
  mkdir $XOC_SYSTEM/cache
  mkdir $XOC_SYSTEM/log

  (
    cd $XOC_SYSTEM/bin
    ln -s $WORKSPACE/services/session/starter/starter starter
  )

  createSystemLoggerConf $XOC_SYSTEM/etc/SystemLogger.conf
  createTestRdbs $XOC_SYSTEM/cache
  createUnorc $XOC_SYSTEM/cache/unorc
}

toolDir=$(dirname $(readlink -f $0) )
PRJ=$(dirname $toolDir)
RHEL5=$(uname -r | grep -q 'el5$' ; echo $? )
if [ "$RHEL5" = "0" ] ; then
  BASE=$PRJ/out.el5
else
  BASE=$PRJ/out
fi
#echo "RHEL5=$RHEL5"
echo "BASE=$BASE"
OBJECTID=ComponentA
PORT=8100
ITERATIONS=1
while [ $# -gt 0 ] ; do
  case $1 in
    (-port) shift; PORT=$1 ;;
    (-n)    shift; ITERATIONS=$1 ;;
    (*)     echo "Unexpected option: $1" ;;
  esac
  shift;
done

if [ -z $WORKSPACE ] ; then
  echo "WORKSPACE not defined, need to run in Zenith development environment" 1>&2
fi
if [ -z $OO_SDK_URE_HOME ] ; then
  echo "OO_SDK_URE_HOME not defined, need to run in Zenith development environment" 1>&2
fi

LD_LIBRARY_PATH=$BASE/lib:$LD_LIBRARY_PATH

export XOC_SYSTEM=$BASE/TestBed
createTestBed

# Run ComponentA as service
#LD_DEBUG=all
#strace -o ./ComponentA.strace.txt -tt 
 $XOC_SYSTEM/bin/starter \
  -unorc file://$XOC_SYSTEM/cache/unorc \
  -server \
  -start "com.myco.ComponentA" \
  -port $PORT &

serverPid=$!
sleep 1

#Run TestComponentA as client
#LD_DEBUG=all
if [ -n "useStarter" ] ; then
  $XOC_SYSTEM/bin/starter \
  -unorc file://$XOC_SYSTEM/cache/unorc \
  -start "com.myco.TestComponentA" \
  -args \
  -n $ITERATIONS \
  -port $PORT \
  -unoobj "com.myco.ComponentA" 2>&1
else
  uno \
  -s com.myco.TestComponentA \
  -ro $OO_SDK_URE_HOME/share/misc/types.rdb \
  -ro $OO_SDK_URE_HOME/share/misc/services.rdb \
  -ro $BASE/bin/TestComponentA.uno.rdb \
  -ro $BASE/bin/ComponentA.uno.rdb \
  -- \
  -n $ITERATIONS \
  -port $PORT \
  -unoobj "com.myco.ComponentA" 2>&1
fi

kill $serverPid
